<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Factory;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use Teknisa\Model\Geral\Entities as GeralEntities;
use Teknisa\Model\Services\Entities as ServicesEntities;
use Teknisa\Model\Services\Entities\FctEscfaturexcom;
use Teknisa\Model\Services\Entities\FctEscfatueditelem;

class SMIntegration 
{
    private $entityManager;

    /** @var Factory\SMIntegration **/
    protected $factorySMIntegration;
    
    public function __construct(EntityManager $entityManager, Factory\SMIntegration $factorySMIntegration)
    {
        $this->entityManager = $entityManager;
        $this->factorySMIntegration = $factorySMIntegration;
    }
    
    public function findSMIntegrationURL()
    {
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        return InstanceProvider::getRepository()->getOmIntegracoes()->getSMIntegrationUrl($nrorg);
    }
    public function findSMParameters($filter)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $smParameters = InstanceProvider::getRepository()->getOmIntegracoes()->findSMParameters($nrorg, $filterString);
        return $smParameters;
    }
    public function findBudgetModel($filter)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $smParameters = InstanceProvider::getRepository()->getOmIntegracoes()->findBudgetModel($filterString);
        return $smParameters;
    }
    public function priceCustomization($filter)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $smParameters = InstanceProvider::getRepository()->getOmIntegracoes()->priceCustomization($filterString);
        return $smParameters;
    }
    
    public function lsFindSMIntegration($filter)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $entityBudgetItemFctItemorcamento = InstanceProvider::getRepository()->getBudgetItemFctItemorcamento()->getBudgetItem($nrorg, $filterString);
        return $entityBudgetItemFctItemorcamento;
    }

    public function lsFindSMIntegrationPrices($filter)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $entityBudgetItemFctItemorcamento = InstanceProvider::getRepository()->getBudgetItemFctItemorcamento()->getBudgetItemPrices($nrorg, $filterString);
        return $entityBudgetItemFctItemorcamento;
    }

    public function findServicoOrca($filter, $page, $pageSize)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $params = array(
            "nrorg" => $nrorg
        );
        $servOrca = InstanceProvider::getRepository()->getOmIntegracoes()->findServicoOrca($params, $filterString, $limit, $offset);
        return $servOrca;
    }
    public function findProductLineServOrca()
    {
        $params = array( "nrorg" => InstanceProvider::getEnvironment()->getCurrentOrganizationId() );
        return InstanceProvider::getRepository()->getOmIntegracoes()->findProductLineServOrca($params);
    }
    public function smIntegrationSaveParameters($nrParametroGeral, $idIntegracaoOm,$urlIntegracaoOm,$idIntegracaoCrm,$urlIntegracaoCrm, $nrOrcaModeloCust, $cdPrecBaseServCust)
    {
        $environment = InstanceProvider::getEnvironment();
        $dtinclusao = new \DateTime;
        $cdoperador = $environment->getCurrentUserId();
        $nrorg = $environment->getCurrentOrganizationId();
        $repositoryFctParametrogeral = InstanceProvider::getRepository()->getFctParametrogeral();
        $entityOmIntegracoes = null;
        $entityFctParametrogeral = $repositoryFctParametrogeral->find(array("nrparametrogeral" => $nrParametroGeral, "nrorg" => $nrorg));
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $entityFctParametrogeral = InstanceProvider::getFactory()->getFctParametrogeral()->save($entityFctParametrogeral, $nrorg, $cdoperador, $dtinclusao, $idIntegracaoOm,$urlIntegracaoOm,$idIntegracaoCrm,$urlIntegracaoCrm, $nrOrcaModeloCust, $cdPrecBaseServCust);
            $this->entityManager->persist($entityFctParametrogeral);
            $this->entityManager->flush();
            $connection->commit();
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function smIntegrationSaveCustomizationParameters($nrParametroGeral, $nrOrcaModeloCust, $dsPrecBaseServCust, $cdservmanucust, $nrtipoorcamentocust, $vrpercmanucust, $clientes)
    {
        $environment = InstanceProvider::getEnvironment();
        $dtinclusao = new \DateTime;
        $cdoperador = $environment->getCurrentUserId();
        $nrorg = $environment->getCurrentOrganizationId();
        $repositoryFctParametrogeral = InstanceProvider::getRepository()->getFctParametrogeral();
        $entityOmIntegracoes = null;
        $entityFctParametrogeral = $repositoryFctParametrogeral->find(array("nrparametrogeral" => $nrParametroGeral, "nrorg" => $nrorg));
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $entityFctParametrogeral = InstanceProvider::getFactory()->getFctParametrogeral()->saveCustomization($entityFctParametrogeral, $nrorg, $cdoperador, $dtinclusao, $nrOrcaModeloCust, $dsPrecBaseServCust, $cdservmanucust, $nrtipoorcamentocust, $vrpercmanucust);
            $this->entityManager->persist($entityFctParametrogeral);
            $this->entityManager->flush();

            /* Insere os clientes marcados que não existiam na tabela OM_CLIENTE_CUSTOMIZACAO */
 
            foreach ($clientes as $cliente){
                /** @var \Teknisa\Model\Teknisa\Entities\OmClienteCustomizacao $cliCustom */
                $cliCustom = InstanceProvider::getRepository()->getOmClienteCustomizacao()->findOneBy(
                    array('cdcliente' => $cliente, 
                           'nrorg' => $nrorg)
                );

                if (empty($cliCustom)) {

                    $insCliCustom = $this->factorySMIntegration->saveCliCustomizacao($cliente);
                    $this->entityManager->persist($insCliCustom);
                    $this->entityManager->flush();
                }
            }

            /* Exclui os clientes desmarcados que não existiam na tabela OM_CLIENTE_CUSTOMIZACAO
            /** @var \Teknisa\Model\Teknisa\Entities\OmClienteCustomizacao[] $cliCustomEx */
            $cliCustomEx = InstanceProvider::getRepository()->getOmClienteCustomizacao()->findBy(
                array('nrorg' => $nrorg)
            );

            foreach ($cliCustomEx as $cli){
                if (!in_array($cli->getCdcliente(), $clientes)){
                    
                    $this->entityManager->remove($cli);
                    $this->entityManager->flush();
                }
            }

            $connection->commit();
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function smIntegrationSaveServiOrca($useLicense, $maintenance, $cdtpmedidaservQT = null, $billingScopeTer = null, $billingScopeEditElem = null)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $environment = InstanceProvider::getEnvironment();
            $actualDate  = new \DateTime();
            $cdoperador  = $environment->getCurrentUserId();
            $nrorg       = $environment->getCurrentOrganizationId();


            $allServOrca = InstanceProvider::getRepository()->getFctServicoOrca()->findBy(array('nrorg' => $nrorg));
            /** @var ServicesEntities\FctServicoorca $servOrca */
            foreach ($allServOrca as $servOrca) {
                $isNull = true;
                foreach($useLicense as $cdservorcaLU){
                    if($servOrca->getCdservorca() == $cdservorcaLU) {
                        $servOrca->setIdlicenusomanut(ServicesEntities\FctServicoorca::ID_LICENCA_USO);
                        $isNull = false;
                    }
                }
                foreach($maintenance as $cdservorcaMA){
                    if($servOrca->getCdservorca() == $cdservorcaMA) {
                        $servOrca->setIdlicenusomanut(ServicesEntities\FctServicoorca::ID_MANUTENCAO);
                        $isNull = false;
                    }
                }
                if($isNull){
                    $servOrca->setIdlicenusomanut(null);
                }
                
                $servOrca->setDtultatu($actualDate);
                $servOrca->setCdoperultatu($cdoperador);
                $servOrca->setNrorgultatu($nrorg);
                $this->entityManager->persist($servOrca);
            }

            // Rotina para alteração dos dados do campo "Escopos de Faturamento que Permitem atualizar os elementos na tela de Organização"
            // No caso, primeiro ele sempre exclui todos os registros da tabela "FCT_ESCFATUEDITELEM" e depois insere novamente com os dados enviados pela requisição. 
            
            $allescFatuEditElem = InstanceProvider::getRepository()->getFctEscfatueditelem()->findBy(array('nrorg' => $nrorg));

            /** @var ServicesEntities\FctEscfatueditelem $fctescFatuEditElem */
            foreach ($allescFatuEditElem as $escFatuEditElem){
                
                $this->entityManager->remove($escFatuEditElem);
                $this->entityManager->flush();
            }
            
            foreach ($billingScopeEditElem as $billingScopeEdit){
                
                $fctescFatuEditElem = new FctescFatuEditElem();

                $fctescFatuEditElem->setNrtipoorcamento((int)$billingScopeEdit);
                $fctescFatuEditElem->setNrorg($nrorg);
                $fctescFatuEditElem->setIdativo('S');
                $fctescFatuEditElem->setDtinclusao($actualDate);
                $fctescFatuEditElem->setCdoperinclusao($cdoperador);
                $fctescFatuEditElem->setNrorginclusao($nrorg);
                
                $this->entityManager->persist($fctescFatuEditElem);
            }

            // Rotina para alteração dos dados do campo "Escopos de Faturamento que Permitem a Exclusão do Elemento na Organização"
            // No caso, primeiro ele sempre exclui todos os registros da tabela "FCT_ESCFATUREXCOM" e depois insere novamente com os dados enviados pela requisição. 
            
            $allescFaturExcOM = InstanceProvider::getRepository()->getFctEscfaturexcom()->findBy(array('nrorg' => $nrorg));
            /** @var ServicesEntities\FctEscfaturexcom $escFaturExcOM */
            foreach ($allescFaturExcOM as $escFaturExcOM){
                
                $this->entityManager->remove($escFaturExcOM);
                $this->entityManager->flush();
            }
            
            foreach ($billingScopeTer as $billingScopeT){
                
                $fctEscFaturExcOM = new FctEscfaturexcom();
                
                $fctEscFaturExcOM->setNrtipoorcamento((int)$billingScopeT);
                $fctEscFaturExcOM->setNrorg($nrorg);
                $fctEscFaturExcOM->setIdativo('S');
                $fctEscFaturExcOM->setDtinclusao($actualDate);
                $fctEscFaturExcOM->setCdoperinclusao($cdoperador);
                $fctEscFaturExcOM->setNrorginclusao($nrorg);
                $this->entityManager->persist($fctEscFaturExcOM);
            }

            $allMedidaServ = InstanceProvider::getRepository()->getFctTipomedidaserv()->findBy(array('nrorg' => $nrorg));
            /** @var ServicesEntities\FctTipomedidaserv $medidaServ */
            foreach ($allMedidaServ as $medidaServ) {
                if($cdtpmedidaservQT && $medidaServ->getCdtpmedidaserv() == $cdtpmedidaservQT) {
                    $medidaServ->setIdqtminima(ServicesEntities\FctTipomedidaserv::IDQTMINIMA_SIM);
                } else {
                    $medidaServ->setIdqtminima(ServicesEntities\FctTipomedidaserv::IDQTMINIMA_NAO);
                }
                $medidaServ->setDtultatu($actualDate);
                $medidaServ->setCdoperultatu($cdoperador);
                $medidaServ->setNrorgultatu($nrorg);
                $this->entityManager->persist($medidaServ);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }
    
    public function smIntegrationSaveURL($id, $url)
    {
        $environment = InstanceProvider::getEnvironment();
        $dtinclusao = new \DateTime;
        $cdoperinclusao = $environment->getCurrentUserId();
        $nrorginclusao = $environment->getCurrentOrganizationId();
        $repositoryOmIntegracoes = InstanceProvider::getRepository()->getOmIntegracoes();
        $entityOmIntegracoes = null;
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($id) {
                $entityOmIntegracoes = $repositoryOmIntegracoes->find($id);
            }
            $entityOmIntegracoes = InstanceProvider::getFactory()->getOmIntegracoes()->save($entityOmIntegracoes, $url, $nrorginclusao, $cdoperinclusao, $dtinclusao);
            $this->entityManager->persist($entityOmIntegracoes);
            $this->entityManager->flush();
            $connection->commit();
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }
    
    public function lsFindServices($filter, $page = null, $pageSize = null)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $nrorg  = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $filterString = Utilities::createQueryFiltered($filter);
        return InstanceProvider::getRepository()->getBudgetItemFctItemorcamento()->getNameServiceSM($filterString, $nrorg, $limit, $offset);
    }
    
    public function findItemprodorca($filter)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        return InstanceProvider::getRepository()->getItemprodorcaFctItemorcamento()->getItem($filterString);
    }
    
    public function finditorcaprod()
    {
        
        return InstanceProvider::getRepository()->getItemprodorcaFctItemorcamento()->finditorcaprod();
    }
    
    public function findservorca()
    {
        
        return InstanceProvider::getRepository()->getItemprodorcaFctItemorcamento()->findservorca();
    }

    public function findBudgetItems($filter, $page = null, $pageSize = null)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $nrorg  = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $filterString = Utilities::createQueryFiltered($filter);
        return InstanceProvider::getRepository()->getBudgetItemFctItemorcamento()->findBudgetItems($filterString, $nrorg, $limit, $offset);
    }
    
    public function smIntegrationImportValues($budgets)
    {
        $environment = InstanceProvider::getEnvironment();
        $nrorg = $environment->getCurrentOrganizationId();
        try {
            foreach ($budgets as $budget) {
                if (!is_null($budget['CDSERVORCA'])) {
                   InstanceProvider::getRepository()->getBudgetItemFctItemorcamento()->updateValueBudgetItem($budget['BUDGETID'], $budget['CDSERVORCA'], $nrorg);
                }
            }
        } catch (Exception $ex) {
            throw $ex;
        }
    }
    
    public function smIntegrationSave($id, $cdServOrca, $budgetItemId)
    {
        $environment = InstanceProvider::getEnvironment();
        $dtinclusao = new \DateTime;
        $cdoperinclusao = $environment->getCurrentUserId();
        $nrorginclusao = $environment->getCurrentOrganizationId();
        $repositoryBudgetItemOrca = InstanceProvider::getRepository()->getBudgetItemFctItemorcamento();
        $entityBudgetItemOrca = null;
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($id) {
                $entityBudgetItemOrca = $repositoryBudgetItemOrca->find($id);
            }
            $entityBudgetItemOrca = InstanceProvider::getFactory()->getBudgetItemFctItemorcamento()->save($entityBudgetItemOrca, $cdServOrca, $budgetItemId, $dtinclusao, $cdoperinclusao, $nrorginclusao);
            $this->entityManager->persist($entityBudgetItemOrca);
            $this->entityManager->flush();
            $connection->commit();
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }
    
    public function saveItemprodorcaFctItemorcamento($cdservorca, $cditorcaprod, $id)
    {
        $environment = InstanceProvider::getEnvironment();
        $dtinclusao = new \DateTime;
        $cdoperinclusao = $environment->getCurrentUserId();
        $nrorginclusao = $environment->getCurrentOrganizationId();
        $itemorcaprodFctItemorcamentoRepository = InstanceProvider::getRepository()->getItemprodorcaFctItemorcamento();
        $entityItemorcaprodFctItemorcamento = $id ? $itemorcaprodFctItemorcamentoRepository->find($id) : null;
        $entityItemorcaprodFctItemorcamento = InstanceProvider::getFactory()->getItemorcaprodFctItemorcamento()->save(
            $entityItemorcaprodFctItemorcamento, $cdservorca, $cditorcaprod, $dtinclusao, $cdoperinclusao, $nrorginclusao
        );
        $this->entityManager->persist($entityItemorcaprodFctItemorcamento);
        $this->entityManager->flush();
    }
    
    public function findTipoOrcamento($filter, $page, $pageSize)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $params = array(
            "nrorg" => $nrorg
        );
        $servOrca = InstanceProvider::getRepository()->getOmIntegracoes()->findTipoOrcamento($params, $filterString, $limit, $offset);
        return $servOrca;
    }
    

    public function findCliente($filter, $page, $pageSize)
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $nrorg = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $params = array(
            "nrorg" => $nrorg
        );
        $cliente = InstanceProvider::getRepository()->getOmIntegracoes()->findCliente($params, $filterString, $limit, $offset);
        return $cliente;
    }

    public function findMoeda($filter, $page = null, $pageSize = null)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $nrorg  = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $filterString = Utilities::createQueryFiltered($filter);
        return InstanceProvider::getRepository()->getBudgetItemFctItemorcamento()->findMoeda($filterString, $nrorg, $limit, $offset);
    }

    public function saveBudgetItemsPrices($id, $budgetItemId, $moeda, $value)
    {
        $environment = InstanceProvider::getEnvironment();
        $dtinclusao = new \DateTime;
        $cdoperinclusao = $environment->getCurrentUserId();
        $nrorg = $environment->getCurrentOrganizationId();
        $repositoryBudgetItemPrices = InstanceProvider::getRepository()->getBudgetItemPrices();
        $entityBudgetItemPrices = null;
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($id) {
                $entityBudgetItemPrices = $repositoryBudgetItemPrices->find($id);
            }
            $entityBudgetItemPrices = InstanceProvider::getFactory()->getBudgetItemPrices()->save($entityBudgetItemPrices, $moeda, $budgetItemId, $value, $dtinclusao, $cdoperinclusao, $nrorg);
            $this->entityManager->persist($entityBudgetItemPrices);
            $this->entityManager->flush();
            $connection->commit();
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function removeBudgetItemsPrices($id)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\BudgetItemPrice $repositoryBudgetItemPrices */
        $repositoryBudgetItemPrices = InstanceProvider::getRepository()->getBudgetItemPrices()->find($id);
        $this->entityManager->remove($repositoryBudgetItemPrices);
        $this->entityManager->flush();
    }

}